classdef Problem20 < ImplicitGSIPProblem
    % p=Problem20 creates an instance of the GSIP problem:
    %
    %       min   x^2
    %
    %       s.t.  0.5y^3-x^2 <=0 for all y in Y(x)
    %             -1 <= x <= 1
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               x^2-y^2<= 0
    %               0 <= y <= 1} 
    
    properties(GetAccess=protected)

        
        nrOfUppLevVars=1;                         
        boundsOnUppLevVars=[-1;1];                 
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=0;                   
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[];                  
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=0;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    

    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=x^2;
        end
        function grad_f=gradObjFun(obj,x)                   
            grad_f=2*x;
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)      
                                                         
            g=0.5*y.^3-x^2;            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     
            n=size(y,1);
            Dx_g=-2*x*ones(n,1);                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=3/2*y.^2;                
        end
               
        
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)           
            u=[x^2-y^2];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)              
            Dy_u=[-2*y];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)              
            Dx_u=[2*x];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)           
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=0;
             ub=1;
        end
                        

        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
                        [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
        methods(Access=private)        
        
        function [lb,ub] = getInterval(obj,x)          
            lb=sqrt(x^2);
            ub=1;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            if x==0
            Dx_lb=0;
            else
                Dx_lb=x/abs(x);
            end
            Dx_ub=[0];
        end
        
    end
    
end